

public class MySortedList extends MyList {
	
	public MySortedList (int capacity){
		super (capacity);
	}
	
	/*
	 * Hinweis zu den beiden add-Methoden:
	 * Leider ist es nicht möglich durch direktes Überschreiben nur eine add(IMyComparable)-Methode zu machen.
	 * Deshalb wirft auch @Override einen Fehler bei der add(IMyComparable)-Methode. 
	 * Wir können aber folgendes machen, um das Problem zu korrigieren:
	 *     - wir überschreiben add(Object) und lassen dort nur IMyComparable-Objekte zu ansonsten werfen wir eine Exception
	 *          -> Wir müssen hier das hinzufügen von IMyComparable-Objekten zulassen, da der Compiler für IMyComparable-Objekte, die aber gerade als Object gecastet sind, die add(Object)-Methode aufrufen würde.  
	 *     - wir überladen die add-Methode durch eine neue add(IMyComparable)-Methode
	 */
	
	@Override 
	public void add (Object obj) throws MyContainerException{ //Ueberschriebene Methode, die nicht funktioniert 
		if(obj instanceof IMyComparable){
			// //super.add wirft eine MyContainerException. Durch die throws-Deklaration wird dann die Exception hier einfach weitergeworfen.
			super.add(obj);
			this.sort ();
		}
		else{
		    throw new MyContainerException ("Objekt obj ist nicht vom erwarteten Typ "+IMyComparable.class.getName()+".");
		}
	}
	
	public void add (IMyComparable obj)throws MyContainerException{//Ueberladene Methode
         //super.add wirft eine MyContainerException. Durch die throws-Deklaration wird dann die Exception hier einfach weitergeworfen. 
		super.add(obj);
		
		this.sort ();
	}
		
	
	//Überschriebene get-Methode 
	//-> wir müssen auch MyContainerException weiterwerfen
	@Override
	public IMyComparable get (int index)throws MyContainerException{
		//super.get wirft eine MyContainerException. Durch die throws-Deklaration wird dann die Exception hier einfach weitergeworfen. 
		return (IMyComparable)super.get(index);
	}
	
	private void sort (){
		for ( int i = 0; i < size(); i++){
			
			for ( int j = size()-1; j >= i+1; j--) 
			{
				//Wir müssen casten. Da wir bei der add-Methode sichergestellt haben, dass nur Objekte des Typs IMyComparable hier vorhanden sind, können wir auf den instanceof-check hier verzichten! 
				IMyComparable casted = (IMyComparable)_objs[j];
				if (casted.compareTo(_objs[j-1]) < 0){
					Object temp = _objs[j];
					_objs [j] = _objs[j-1];
					_objs[j-1] = temp;
				}
			}
		}
	}
	
	public static void main (String[] args){
		TestKlasse_Auto a = new TestKlasse_Auto("Mercedes", 2100.0f, 20000); 
		TestKlasse_Auto b = new TestKlasse_Auto("Audi", 1900.0f, 0); 
		TestKlasse_Auto c = new TestKlasse_Auto("Peugeot",  2900.0f, 370); 
		TestKlasse_Fahrrad d = new TestKlasse_Fahrrad("Bianchi",  10); 
		MySortedList test = new MySortedList (8);
		try {
			//Wir müssen jetzt hier ein catch machen, weil MyContainerException geworfen wird und das in der main-Methode gefangen werden muss!
			test.add(a);
			test.add(b);
			test.add(c);
			test.add(d);
		
			System.out.println("Meine Liste ist sortiert nach Kilometerstand: " + test.toString());
		}catch (MyContainerException mcex){
			mcex.printStackTrace();
		}
	}
	
}
