
/**
 * Hier eine einfache Klasse "Fahrzeug" zum Testen der Container-Klassen. 
 * Sie dient als Basisklasse für "Fahrrad" und "Auto".
 * Wir können hier IMyComparable definieren, um auch zwischen Fahrrad und Auto eine Vergleichbarkeit zu haben.
 *
 * Weiterer Hinweis: Diese Testklassen sollten eigentlich nicht im selben Order/Paket liegen wie
 * die zu entwickelnden Klassen, da sie nur zu Testzwecken existieren. Das besprechen wir aber 
 * dann erst im 2. Semester bei PM nochmals richtig. 
 */
public abstract class TestKlasse_Fahrzeug implements IMyComparable{
	
	protected String _name;
	protected  float _kilometerstand;
	protected boolean _belegt;
	
	public TestKlasse_Fahrzeug (String name, float kilometerstand) {
		this._name = name;
		this._kilometerstand = kilometerstand;
		this._belegt = false;
	}
	
	public String getName (){
		return _name;
	}
	
	public float getKilometerstand (){
		return _kilometerstand;
	}
	
	public boolean getBelegt (){
		return _belegt;
	}
	
	
	
	public abstract int maxReichweite (); // Wird dann erst überschrieben
	
	public void belege(){
		_belegt = true;
	}
	
	public void gebeFrei (float gefahreneKm){
		_belegt = false;
		this._kilometerstand = this._kilometerstand + gefahreneKm;
	}
	
	public String toString (){	
		return this.getClass().getName() + " " + getName() + ", Km: " + getKilometerstand() + ", Belegt: " + this.getBelegt(); 
	}
		
	//CompareTo-Methode ist jetzt nötig, weil ja IMyComparable implementiert werden muss
	// Hier ist wichtig sich zu entscheiden nach welchem Kriterium überhaupt soritert werden soll. Soll es der Name sein? Soll es der Kilometerstand sein? ... -> Wir machen Kilometerstand.
	 public int compareTo (Object obj){
		 if (obj instanceof TestKlasse_Fahrzeug) {
			 //Wir können jetzt z.B. nach Kilometerstand sortieren:
			 
			 /* Entweder so - aber ganz schön kompliziert:
			 if (this.getKilometerstand() < ((TestKlasse_Fahrzeug)obj).getKilometerstand()){
				 return -1;
			 }
			 else if (this.getKilometerstand() == ((TestKlasse_Fahrzeug)obj).getKilometerstand()){
				 return 0;
			 }	 
			 else {
				 return 1;
			}
			*/
			
			//Oder wir nutzen hier den Vorteil, dass Kilometerstand eine Zahl ist
			return (int) (this.getKilometerstand() - ((TestKlasse_Fahrzeug)obj).getKilometerstand());
			
			// ABER VORSICHT: Wir rechnen auf float und der cast zu int würde bei den Zahlen -1< x <1 auf 0 sein. In diesem Fall ist das vertretbar, in anderen vielleicht nicht.
			
	    }
		else{
			 //Wenn wir es nicht vergleichen können, werfen wir eine Exception
			 throw new RuntimeException("Nicht möglich Fahrzeug '"+this.toString()+"' mit Objekt '"+obj.toString()+"' zu vergleichen");
			 //Alternativ könnten wir auch die toString()-Ergebnisse vergleichen:
			 //return this.toString().compareTo(obj.toString());
		}
	 }
}



